function [obj] = sub_objdec(arg)

% ================================================================================= % 
% opt_TR_SW_objdec.m 
%
% Here we decompose the objective into two parts. One part reflects the
% difference of actual from estimates and the second part relects the
% minimization of the estimates.
%
% Chris Boehm 
% March 2014
% ================================================================================= % 

global A11 A21 A22 Etild SIGuu G D1 D2 P altrho_u alpha kappa sig beta 

opt = optimset('TolFun',eps,'TolX',eps,'Display','off');
% rand('seed',1111) 

% Model Parameters 

% Preferences

beta        = 0.99;     % discount factor
theta       = 2/3;      % price stickyness

sig         = 1;        % coefficient of relative risk aversion
eta         = 1;        % labor supply elasticity

kappa       = (1-theta)*(1-theta*beta)/theta*(sig + 1/eta);   %macroeconomic rate of price adjustment



% Shock Processes

altrho_re   = 0.9;%0.9;     % persistence of the efficient rate shock
altrho_u    = 0.5;%0.5;      % persistence of the cost-push shock
altrho_my   = 0.95;%0.95;      % persistence of the measurement error in y
altrho_mpi  = 0.0; %0.0;     % persistence of the measurement error in pi

sig_re      = 0.22;%0.22;        % standard deviation of the natural rate shock
sig_u       = 0.43;%0.43;        % standard deviation of the cost-push shock
sig_my      = 0.66;%0.66;      % standard deviation of the measurement error in y
sig_mpi     = 0.50;%0.50;      % standard deviation of the measurement error in pi

% Welfare criterion

alpha = 1;          % weight on inflation (the weight on the gap is unity)

% Taylor rule

% reaction on output gap (infinity is optimal,
% but then then the algorithm breaks down due to noninvertibilities.
psiy  = arg(1);%9500;      

% reaction on inflation on the manyfold
psipi = arg(2);
% psipi = altrho_u + alpha*kappa*sig*(1-altrho_u)/(1-beta*altrho_u) ...
%     + alpha*kappa/(1-beta*altrho_u)*psiy;    


%==========================================================================
% Computing the coefficient matrices

A = zeros(6,6);

A(1,1) = altrho_re;
A(2,2) = altrho_u;
A(3,3) = altrho_my;
A(4,4) = altrho_mpi;
A(5,2) = -1/beta;
A(5,5) = -kappa/beta;
A(5,6) = 1/beta;
A(6,1) = -1;
A(6,5) = sig;

A11 = A(1:4,1:4);
A21 = A(5:6,1:4);
A22 = A(5:6,5:6);

Etild = [ 0 1; sig 1];

SIGuu = zeros(4,4);
SIGuu(1,1) = sig_re^2;
SIGuu(2,2) = sig_u^2;
SIGuu(3,3) = sig_my^2;
SIGuu(4,4) = sig_mpi^2;

D = [zeros(2) eye(2) eye(2)];

D1 = D(:,1:4);
D2 = D(:,5:6);

G = zeros(2,4);

G(1,1) = (1-beta*altrho_re)/((sig*(1-altrho_re) + psiy)*(1-beta*altrho_re) + kappa*(psipi - altrho_re));
G(2,1) = kappa/((sig*(1-altrho_re) + psiy)*(1-beta*altrho_re) + kappa*(psipi - altrho_re));

G(1,2) = (altrho_u - psipi)/((sig*(1-altrho_u) + psiy)*(1-beta*altrho_u) + kappa*(psipi - altrho_u));
G(2,2) = (sig*(1-altrho_u) + psiy)/((sig*(1-altrho_u) + psiy)*(1-beta*altrho_u) + kappa*(psipi - altrho_u));

G1startval = [2.0701   -0.1451   -0.6619         0;...
            1.2341    1.3531   -0.5575         0];

SOL = fsolve('obj_sw',G1startval,opt);

G1(1:2,1) = SOL(1:2);
G1(1:2,2) = SOL(3:4);
G1(1:2,3) = SOL(5:6);
G1(1:2,4) = SOL(7:8);

L = D1 + D2*G1;
M = D2*(G-G1);
K = P*L'*(L*P*L')^(-1);



%******************************************************************
%**** Begin setup for constructing log-linear system coef matrix **
%******************************************************************


nlead = 1;  % Number of leads in system 
nlag  = 1;  % Number of lags in system 

xnum = 15+4; 	%  Number of variables in system (plus shocks) 
neq = xnum;	    %  Number of equations (same)

% =========================================================================

colzero = 0+nlag*xnum;      % Position counter for start of contemp. coefs 
                            % i.e. if there is one lag and 5 equations then the "first"
                            % contemporaneous variable is at position 6 (6 = 1*5+cpos)
collead = 0+nlag*xnum+xnum; % Position counter for start of lead coefs 
collag  = 0;                % Position counter for start of lag coefs  

% Position counters

ypos        = 1;
pipos       = 2;
repos       = 3;
upos        = 4;
mypos       = 5;
mpipos      = 6;
y_est_pos   = 7;
pi_est_pos  = 8;
re_est_pos  = 9;
u_est_pos   = 10;
mpi_est_pos = 11;
my_est_pos  = 12;
y_m_pos     = 13;
pi_m_pos    = 14;
ipos        = 15;
e_repos     = 16;
e_upos      = 17;
e_mypos     = 18;
e_mpipos    = 19;

% Indicators for contemporanous coefs for each variable: 

yzero           = colzero + ypos;
pizero          = colzero + pipos;
rezero          = colzero + repos;
uzero           = colzero + upos;
myzero          = colzero + mypos;
mpizero         = colzero + mpipos;
y_est_zero      = colzero + y_est_pos;
pi_est_zero     = colzero + pi_est_pos;
re_est_zero     = colzero + re_est_pos;
u_est_zero      = colzero + u_est_pos;
mpi_est_zero    = colzero + mpi_est_pos;
my_est_zero     = colzero + my_est_pos;
y_m_zero        = colzero + y_m_pos;
pi_m_zero       = colzero + pi_m_pos;
izero           = colzero + ipos;
e_rezero        = colzero + e_repos;
e_uzero         = colzero + e_upos;
e_myzero        = colzero + e_mypos;
e_mpizero       = colzero + e_mpipos;

% Indicators for lead coefficients for each variable: 

ylead           = collead + ypos;
pilead          = collead + pipos;
relead          = collead + repos;
ulead           = collead + upos;
mylead          = collead + mypos;
mpilead         = collead + mpipos;
y_est_lead      = collead + y_est_pos;
pi_est_lead     = collead + pi_est_pos;
re_est_lead     = collead + re_est_pos;
u_est_lead      = collead + u_est_pos;
mpi_est_lead    = collead + mpi_est_pos;
my_est_lead     = collead + my_est_pos;
y_m_lead        = collead + y_m_pos;
pi_m_lead       = collead + pi_m_pos;
ilead           = collead + ipos;
e_relead        = collead + e_repos;
e_ulead         = collead + e_upos;
e_mylead        = collead + e_mypos;
e_mpilead       = collead + e_mpipos;

% Indicators for lag coefficients for each variable: 

ylag           = collag + ypos;
pilag          = collag + pipos;
relag          = collag + repos;
ulag           = collag + upos;
mylag          = collag + mypos;
mpilag         = collag + mpipos;
y_est_lag      = collag + y_est_pos;
pi_est_lag     = collag + pi_est_pos;
re_est_lag     = collag + re_est_pos;
u_est_lag      = collag + u_est_pos;
mpi_est_lag    = collag + mpi_est_pos;
my_est_lag     = collag + my_est_pos;
y_m_lag        = collag + y_m_pos;
pi_m_lag       = collag + pi_m_pos;
ilag           = collag + ipos;
e_relag        = collag + e_repos;
e_ulag         = collag + e_upos;
e_mylag        = collag + e_mypos;
e_mpilag       = collag + e_mpipos;


% NOTE: these are all vectors of position counters
% Now I have one vector with all of the leads, lags, etc in one column

% Determine number of coefficients per equation: 

ncoef = neq*(nlag+nlead+1);

cof = zeros(neq,ncoef);             % Coef matrix --- Each row is an equation

% ================================================================================= % 
% ================================================================================= % 
% Fill in the coefficient matrix (cof)
% ================================================================================= % 
% ================================================================================= % 

% ================================================================================= % 
% Equation (1) Shock process of the efficient rate
% ================================================================================= % 

  cof(1,rezero)   = -1;
  cof(1,relag)    = altrho_re;
  cof(1,e_rezero) = 1;
  
% ================================================================================= % 
% Equation (2) Shock process of the cost-push shock
% ================================================================================= % 

  cof(2,uzero)   = -1;
  cof(2,ulag)    = altrho_u;
  cof(2,e_uzero) = 1;
  
% ================================================================================= % 
% Equation (3) Shock process of measurement error in y
% ================================================================================= % 

  cof(3,myzero)   = -1;
  cof(3,mylag)    = altrho_my;
  cof(3,e_myzero) = 1;
  
% ================================================================================= % 
% Equation (4) Shock process of measurement error in pi
% ================================================================================= % 

  cof(4,mpizero)   = -1;
  cof(4,mpilag)    = altrho_mpi;
  cof(4,e_mpizero) = 1;
  
% ================================================================================= % 
% Equation (5-8) Evolution of the estimates of the shocks
% ================================================================================= %

OM1 = (eye(4) + K*M)^(-1)*(eye(4) - K*L)*A11;
OM2 = (eye(4) + K*M)^(-1)*K;

  cof(5,re_est_zero)    = -1;
  cof(5,re_est_lag)     = OM1(1,1);
  cof(5,u_est_lag)      = OM1(1,2);
  cof(5,my_est_lag)     = OM1(1,3);
  cof(5,mpi_est_lag)    = OM1(1,4);
  cof(5,y_m_zero)        = OM2(1,1);
  cof(5,pi_m_zero)       = OM2(1,2);
  
  cof(6,u_est_zero)    = -1;
  cof(6,re_est_lag)     = OM1(2,1);
  cof(6,u_est_lag)      = OM1(2,2);
  cof(6,my_est_lag)     = OM1(2,3);
  cof(6,mpi_est_lag)    = OM1(2,4);
  cof(6,y_m_zero)       = OM2(2,1);
  cof(6,pi_m_zero)      = OM2(2,2);  
  
  cof(7,my_est_zero)    = -1;
  cof(7,re_est_lag)     = OM1(3,1);
  cof(7,u_est_lag)      = OM1(3,2);
  cof(7,my_est_lag)     = OM1(3,3);
  cof(7,mpi_est_lag)    = OM1(3,4);
  cof(7,y_m_zero)       = OM2(3,1);
  cof(7,pi_m_zero)      = OM2(3,2); 
  
  cof(8,mpi_est_zero)   = -1;
  cof(8,re_est_lag)     = OM1(4,1);
  cof(8,u_est_lag)      = OM1(4,2);
  cof(8,my_est_lag)     = OM1(4,3);
  cof(8,mpi_est_lag)    = OM1(4,4);
  cof(8,y_m_zero)       = OM2(4,1);
  cof(8,pi_m_zero)      = OM2(4,2);  
  
% ================================================================================= % 
% Equation (9-10) Measures of output and inflation
% ================================================================================= %   
  
  cof(9,y_m_zero)       = -1;
  cof(9,yzero)          = 1; 
  cof(9,myzero)         = 1; 
  
  cof(10,pi_m_zero)     = -1;
  cof(10,pizero)        = 1; 
  cof(10,mpizero)        = 1;  

% ================================================================================= % 
% Equation (11 - 12) Estimates of output and inflation
% ================================================================================= % 

  cof(11,y_est_zero)    = -1;
  cof(11,re_est_zero)   = G(1,1);
  cof(11,u_est_zero)    = G(1,2);
  cof(11,my_est_zero)   = G(1,3);
  cof(11,mpi_est_zero)  = G(1,4);
  
  cof(12,pi_est_zero)   = -1;
  cof(12,re_est_zero)   = G(2,1);
  cof(12,u_est_zero)    = G(2,2);
  cof(12,my_est_zero)   = G(2,3);
  cof(12,mpi_est_zero)  = G(2,4);  
     
% ================================================================================= % 
% Equation (13-14) Actual output and inflation
% ================================================================================= % 

  cof(13,yzero)         = -1;
  cof(13,rezero)        = G1(1,1);
  cof(13,uzero)         = G1(1,2);
  cof(13,myzero)        = G1(1,3);
  cof(13,mpizero)       = G1(1,4);   
  cof(13,re_est_zero)   = G(1,1) - G1(1,1);
  cof(13,u_est_zero)    = G(1,2) - G1(1,2);
  cof(13,my_est_zero)   = G(1,3) - G1(1,3);
  cof(13,mpi_est_zero)  = G(1,4) - G1(1,4); 
  
  cof(14,pizero)        = -1;
  cof(14,rezero)        = G1(2,1);
  cof(14,uzero)         = G1(2,2);
  cof(14,myzero)        = G1(2,3);
  cof(14,mpizero)       = G1(2,4);   
  cof(14,re_est_zero)   = G(2,1) - G1(2,1);
  cof(14,u_est_zero)    = G(2,2) - G1(2,2);
  cof(14,my_est_zero)   = G(2,3) - G1(2,3);
  cof(14,mpi_est_zero)  = G(2,4) - G1(2,4);   
    
% ================================================================================= % 
% Equation (15) Taylor rule
% ================================================================================= %     

  cof(15,izero)             = -1;
  cof(15,y_est_zero)        = psiy; 
  cof(15,pi_est_zero)       = psipi;
 
% ================================================================================= % 
% Equations (16-19) Shocks
% ================================================================================= % 

  cof(16,e_rezero)      = 1;
  
  cof(17,e_uzero)       = 1;
  
  cof(18,e_myzero)      = 1;
  
  cof(19,e_mpizero)     = 1;

%*****************************************************************
%*************Begin Solution Algorithm ***************************
%*****************************************************************
%                                                                 
%  Solve a linear perfect foresight model using the gauss eig     
%  function to find the invariant subspace associated with the big 
%  roots.  This procedure will fail if the companion matrix is     
%  defective and does not have a linearly independent set of       
%  eigenvectors associated with the big roots.                     
%                                                                  
%  Input arguments:                                                
%                                                                  
%    h         Structural coef matrix (neq,neq*(nlag+1+nlag)).     
%    neq       Number of equations.                                
%    nlag      Number of lags.                                                                  
%    condn     lag tolerance used as a condition number test       
%              by numeric_shift and reduced_form.                  
%    uprbnd    Inclusive upper bound for the modulus of roots      
%              allowed in the reduced form.                        
%                                                                  
%  Output arguments:                                               
%                                                                  
%    cofb      Reduced form coefficient matrix (neq,neq*nlag).     
%    scof      Observable Structure                                
%    amat      Companion form matrix                               
%    b         Contemporaneous coefficient matrix                  
%    Model satisfies:                                              
%                                                                  
%    z(t) = amat*z(t-1) + b*e(t)                                   
%                                                                  
%    where the first neq elements of z(t) are the contemporaneous  
%    values of the variables in the model                          
%    and e(t) is the shock vector of conformable dimension         
%                                                                  
%    rts       Roots returned by eig.                              
%    ia        Dimension of companion matrix (number of non-trivial
%              elements in rts).                                   
%    nexact    Number of exact shiftrights.                        
%    nnumeric  Number of numeric shiftrights.                      
%    lgroots   Number of roots greater in modulus than uprbnd.     
%    mcode     Return code: see function aimerr.                   
%*****************************************************************


% Use AIM procedure to solve model: 
uprbnd = 1+1e-8;    % Tolerance values for AIM program 
condn = 1e-8;

% ---------------------------------------------------------------------
% Run AIM
% ---------------------------------------------------------------------

[cofb,rts,ia,nex,nnum,lgrts,mcode] = ...
       aim_eig(cof,neq,nlag,nlead,condn,uprbnd);

scof = obstruct(cof,cofb,neq,nlag,nlead);

% need to calculate amat and b
% ===============================
s0 = scof(:,(neq*nlag+1):neq*(nlag+1)); 	%Contemp. coefs from obs. structure
amat=zeros(neq*nlag,neq*nlag);   		% Initialize A matrix 
bmat=cofb(1:neq,((nlag-1)*neq+1):nlag*neq);  	% Lag 1 coefficients 
i=2;
while i<=nlag;
  bmat=[bmat cofb(1:neq,((nlag-i)*neq+1):(nlag-i+1)*neq)]; % Lag i coefs 
  i=i+1;
end;
amat(1:neq,:)=bmat;  				% Coefs for equations 
if nlag>1;
 amat((length(cofb(:,1))+1):length(amat(:,1)),1:neq*(nlag-1))=eye(neq*(nlag-1));
end;
b = zeros(length(amat(:,1)),length(s0(1,:)));
b(1:length(s0(:,1)),1:length(s0(1,:))) = inv(s0);  % Store coefs 
%b=b(:,shockvec);

% ================================================================================= % 
% Impulse Response Function
%
% ================================================================================= % 
% Get VAR form

AVAR = amat;
BVAR = b; 

%Variance and Autocorrelations

om = zeros(neq,neq);
om(e_repos,e_repos)     = sig_re^2;
om(e_upos,e_upos)       = sig_u^2;
om(e_mpipos,e_mpipos)   = sig_mpi^2;
om(e_mypos,e_mypos)     = sig_my^2;

V = cvar(AVAR,BVAR,om);            % Analytical way of solving for the covariance matrix

%==========================================================================
W = [1 0; 0 alpha];

scndterm = trace(P*(eye(4)-K*L)'*G1'*W*G1*(eye(4)-K*L));

frstterm = V(y_est_pos,y_est_pos)+alpha*V(pi_est_pos,pi_est_pos);

% disp('First and second term')
% disp([frstterm scndterm])

obj = [frstterm; scndterm;(frstterm+scndterm)];
end
  